@@ -1,21 +1,22 @@
-mod completions;
 mod parallel;
 
 use anyhow::{anyhow, bail, Context, Result};
 use argc::{
-    utils::{escape_shell_words, get_current_dir, get_shell_path, is_true_value, termwidth},
-    Shell,
+    compgen_kind,
+    utils::{escape_shell_words, is_true_value},
+    CompKind, NativeRuntime, Runtime, Shell, COMPGEN_KIND_SYMBOL,
 };
 use base64::{engine::general_purpose, Engine as _};
+use path_absolutize::Absolutize;
 use std::{
     collections::HashMap,
-    env, fs,
+    env,
+    ffi::OsStr,
+    fs,
     path::{Path, PathBuf},
     process,
 };
-use which::which;
 
-const COMPLETION_SCRIPT: &str = include_str!("completions/completion.sh");
 const ARGC_SCRIPT_NAMES: [&str; 6] = [
     "Argcfile.sh",
     "Argcfile",
@@ -25,6 +26,8 @@ const ARGC_SCRIPT_NAMES: [&str; 6] = [
     "ARGCFILE",
 ];
 
+const ARGC_COMPLETION_SCRIPT: &str = include_str!("completion.sh");
+
 fn main() {
     match run() {
         Ok(code) => {
@@ -33,13 +36,14 @@ fn main() {
             }
         }
         Err(err) => {
-            eprintln!("{}", err);
+            eprintln!("{err:?}");
             process::exit(1);
         }
     }
 }
 
 fn run() -> Result<i32> {
+    let runtime = NativeRuntime;
     let args: Vec<String> = std::env::args().collect();
     let mut argc_cmd = None;
     if let Some(arg) = args.get(1) {
@@ -51,45 +55,82 @@ fn run() -> Result<i32> {
     if let Some(argc_cmd) = argc_cmd {
         match argc_cmd {
             "--argc-eval" => {
-                let (source, cmd_args) = parse_script_args(&args[2..])?;
+                let (source, _script_path, cmd_args) = parse_script_args(&args[2..])?;
                 if cmd_args
                     .get(1)
-                    .map(|v| v == parallel::PARALLEL_MODE)
+                    .map(|v| v == parallel::PARALLEL_SYMBOL)
                     .unwrap_or_default()
                 {
                     let cmd_args_len = cmd_args.len();
                     if cmd_args_len < 3 {
                         bail!("No parallel command")
                     }
-                    let mut code = retrive_argc_variables().unwrap_or_default();
+                    let mut code = retrieve_argc_variables().unwrap_or_default();
                     let mut cmds = vec![cmd_args[2].to_string()];
                     cmds.extend(cmd_args[3..].iter().map(|v| escape_shell_words(v)));
                     code.push_str(&cmds.join(" "));
                     println!("{code}")
                 } else {
-                    let values = argc::eval(&source, &cmd_args, Some(&args[2]), termwidth())?;
+                    let values = argc::eval(
+                        runtime,
+                        &source,
+                        &cmd_args,
+                        Some(&args[2]),
+                        get_term_width(),
+                    )?;
                     let dir_vars = export_dir_vars(&args[2]);
-                    let code = argc::ArgcValue::to_shell(&values);
+                    let code = argc::ArgcValue::to_bash(&values);
                     let export_vars = export_argc_variables(&code);
                     println!("{dir_vars}{export_vars}{code}")
                 }
             }
+            "--argc-run" => {
+                if args.len() < 3 {
+                    bail!("No script file provided");
+                }
+                let shell = runtime.shell_path()?;
+                let script_path = normalize_script_path(&args[2]);
+                let (script_dir, script_path) = {
+                    let absolute_script_path = Path::new(&script_path)
+                        .absolutize()
+                        .with_context(|| format!("Invalid script path '{script_path}'"))?;
+                    let script_dir = absolute_script_path.parent().ok_or_else(|| {
+                        anyhow!("Unable to retrieve the script dir from '{script_path}'")
+                    })?;
+                    (
+                        script_dir.to_path_buf(),
+                        absolute_script_path.display().to_string(),
+                    )
+                };
+                let mut envs = HashMap::new();
+                let cwd = if is_runner_script(&script_path) {
+                    if let Some(cwd) = runtime.current_dir() {
+                        if env::var("ARGC_PWD").is_err() {
+                            envs.insert("ARGC_PWD".to_string(), escape_shell_words(&cwd));
+                        }
+                    }
+                    Some(script_dir.as_path())
+                } else {
+                    None
+                };
+                let args = [vec![&script_path], args.iter().skip(3).collect()].concat();
+                return run_command(&script_path, &shell, &args, envs, cwd);
+            }
             "--argc-create" => {
                 if let Some((_, script_file)) = get_script_path(false) {
                     bail!("Already exist {}", script_file.display());
                 }
                 let content = generate_boilerplate(&args[2..]);
-                let names = candidate_script_names();
+                let names = runner_script_names();
                 fs::write(&names[0], content)
                     .with_context(|| format!("Failed to create {}", &names[0]))?;
                 println!("{} has been successfully created.", &names[0]);
             }
             "--argc-build" => {
-                let script_path = &args[2];
-                let (source, args) = parse_script_args(&args[2..])?;
-                let script = argc::build(&source, &args[0])?;
-                if let Some(outpath) = args.get(1) {
-                    let script_name = get_script_name(script_path)?;
+                let (source, script_path, cmd_args) = parse_script_args(&args[2..])?;
+                let script = argc::build(&source, &cmd_args[0], get_term_width())?;
+                if let Some(outpath) = cmd_args.get(1) {
+                    let script_name = get_script_name(&script_path)?;
                     let (outpath, new) = ensure_outpath(outpath, script_name)
                         .with_context(|| format!("Invalid output path '{outpath}'"))?;
                     fs::write(&outpath, script).with_context(|| {
@@ -108,9 +149,9 @@ fn run() -> Result<i32> {
                 }
             }
             "--argc-mangen" => {
-                let (source, args) = parse_script_args(&args[2..])?;
-                let outdir = args.get(1).ok_or_else(|| anyhow!("No output dir"))?;
-                let pages = argc::mangen(&source, &args[0])?;
+                let (source, _script_path, cmd_args) = parse_script_args(&args[2..])?;
+                let outdir = cmd_args.get(1).ok_or_else(|| anyhow!("No output dir"))?;
+                let pages = argc::mangen(&source, &cmd_args[0])?;
                 let outdir = ensure_outdir(outdir).with_context(|| "Invalid output dir")?;
                 for (filename, page) in pages {
                     let outfile = outdir.join(filename);
@@ -124,28 +165,28 @@ fn run() -> Result<i32> {
                     Some(v) => v.parse()?,
                     None => bail!("Usage: argc --argc-completions <SHELL> [CMDS]..."),
                 };
-                let script = crate::completions::generate(shell, &args[3..]);
+                let commands = [vec!["argc".to_string()], args[3..].to_vec()].concat();
+                let script = argc::generate_completions(shell, &commands);
                 print!("{}", script);
             }
             "--argc-compgen" => {
-                run_compgen(args.to_vec());
+                run_compgen(runtime, args.to_vec());
             }
             "--argc-export" => {
-                let (source, args) = parse_script_args(&args[2..])?;
-                let value = argc::export(&source, &args[0])?;
+                let (source, _script_path, cmd_args) = parse_script_args(&args[2..])?;
+                let value = argc::export(&source, &cmd_args[0])?;
                 println!("{}", serde_json::to_string_pretty(&value)?);
             }
             "--argc-parallel" => {
                 if args.len() <= 3 {
                     bail!("Usage: argc --argc-parallel <SCRIPT> <ARGS>...");
                 }
-                let shell = get_shell_path()?;
-                let (source, cmd_args) = parse_script_args(&args[2..])?;
+                let shell = runtime.shell_path()?;
+                let (source, script_path, cmd_args) = parse_script_args(&args[2..])?;
                 if !source.contains("--argc-eval") {
                     bail!("Parallel only available for argc based scripts.")
                 }
-                let script_file = args[2].clone();
-                parallel::parallel(&shell, &script_file, &cmd_args[1..])?;
+                parallel::parallel(runtime, &shell, &script_path, &cmd_args[1..])?;
             }
             "--argc-script-path" => {
                 let (_, script_file) =
@@ -153,11 +194,11 @@ fn run() -> Result<i32> {
                 println!("{}", script_file.display());
             }
             "--argc-shell-path" => {
-                let shell = get_shell_path()?;
-                println!("{}", shell.display());
+                let shell = runtime.shell_path()?;
+                println!("{}", shell);
             }
             "--argc-help" => {
-                println!("{}", get_argc_help())
+                println!("{}", get_argc_help(runtime)?)
             }
             "--argc-version" => {
                 println!("{}", get_argc_version())
@@ -168,36 +209,49 @@ fn run() -> Result<i32> {
         }
         Ok(0)
     } else {
-        let shell = get_shell_path()?;
+        let shell = runtime.shell_path()?;
         let (script_dir, script_file) = get_script_path(true)
             .ok_or_else(|| anyhow!("Argcfile not found, try `argc --argc-help` for help."))?;
         let mut envs = HashMap::new();
-        if let Some(cwd) = get_current_dir() {
-            envs.insert("ARGC_PWD".to_string(), escape_shell_words(&cwd));
-        }
-        let mut command = process::Command::new(shell);
-        command
-            .arg(&script_file)
-            .args(&args[1..])
-            .current_dir(script_dir)
-            .envs(envs);
-        #[cfg(unix)]
-        {
-            use std::os::unix::process::CommandExt;
-            let err = command.exec();
-            bail!("Failed to run `{err}`");
-        }
-        #[cfg(not(unix))]
-        {
-            let status = command
-                .status()
-                .with_context(|| format!("Failed to run `{}`", script_file.display()))?;
-            Ok(status.code().unwrap_or_default())
+        if let Some(cwd) = runtime.current_dir() {
+            if env::var("ARGC_PWD").is_err() {
+                envs.insert("ARGC_PWD".to_string(), escape_shell_words(&cwd));
+            }
         }
+        let script_file = script_file.display().to_string();
+        let args = [vec![&script_file], args[1..].iter().collect()].concat();
+        run_command(&script_file, &shell, &args, envs, Some(&script_dir))
     }
 }
 
-fn run_compgen(mut args: Vec<String>) -> Option<()> {
+fn run_command<T: AsRef<OsStr>>(
+    script_path: &str,
+    prog: &str,
+    args: &[T],
+    envs: HashMap<String, String>,
+    cwd: Option<&Path>,
+) -> Result<i32> {
+    let mut command = process::Command::new(prog);
+    command.args(args).envs(envs);
+    if let Some(cwd) = cwd {
+        command.current_dir(cwd);
+    }
+    #[cfg(unix)]
+    {
+        use std::os::unix::process::CommandExt;
+        let err = command.exec();
+        bail!("Failed to run '{script_path}', {err}");
+    }
+    #[cfg(not(unix))]
+    {
+        let status = command
+            .status()
+            .with_context(|| format!("Failed to run '{script_path}'"))?;
+        Ok(status.code().unwrap_or_default())
+    }
+}
+
+fn run_compgen(runtime: NativeRuntime, mut args: Vec<String>) -> Option<()> {
     if args.len() < 6 {
         return None;
     };
@@ -209,22 +263,48 @@ fn run_compgen(mut args: Vec<String>) -> Option<()> {
             }
         } else if let Some(script_file) = search_completion_script(&mut args) {
             args[3] = script_file.to_string_lossy().to_string();
-        } else if let Ok(script_file) = which(&args[4]) {
-            args[3] = script_file.to_string_lossy().to_string();
+        } else if let Some(script_file) = runtime.which(&args[4]) {
+            args[3] = script_file;
         }
     }
     let no_color = std::env::var("NO_COLOR")
         .map(|v| is_true_value(&v))
         .unwrap_or_default();
+    let last_arg = args.last().map(|v| v.as_str()).unwrap_or_default();
     let output = if &args[4] == "argc" && (args[3].is_empty() || args[5].starts_with("--argc")) {
-        let cmd_args = &args[4..];
-        argc::compgen(shell, "", COMPLETION_SCRIPT, cmd_args, no_color).ok()?
+        if (args[5] == "--argc-completions" || args[5] == "--argc-compgen") && args.len() == 7 {
+            compgen_kind(runtime, shell, CompKind::Shell, last_arg, no_color).ok()?
+        } else if args[5] == "--argc-compgen" {
+            compgen_kind(runtime, shell, CompKind::Path, last_arg, no_color).ok()?
+        } else {
+            argc::compgen(
+                runtime,
+                shell,
+                "",
+                ARGC_COMPLETION_SCRIPT,
+                &args[4..],
+                no_color,
+            )
+            .ok()?
+        }
+    } else if args[3] == COMPGEN_KIND_SYMBOL {
+        let kind = CompKind::new(&args[4]);
+        compgen_kind(runtime, shell, kind, last_arg, no_color).ok()?
     } else if args[3].is_empty() {
-        let cmd_args = &args[4..];
-        argc::compgen(shell, "", "# @arg path*", cmd_args, no_color).ok()?
+        // Fallback unknown command to path completion
+        let last_arg = args.last().map(|v| v.as_str()).unwrap_or_default();
+        compgen_kind(runtime, shell, CompKind::Path, last_arg, no_color).ok()?
     } else {
-        let (source, cmd_args) = parse_script_args(&args[3..]).ok()?;
-        argc::compgen(shell, &args[3], &source, &cmd_args[1..], no_color).ok()?
+        let (source, script_path, cmd_args) = parse_script_args(&args[3..]).ok()?;
+        argc::compgen(
+            runtime,
+            shell,
+            &script_path,
+            &source,
+            &cmd_args[1..],
+            no_color,
+        )
+        .ok()?
     };
     if !output.is_empty() {
         println!("{output}");
@@ -250,36 +330,40 @@ fn export_argc_variables(code: &str) -> String {
     )
 }
 
-fn retrive_argc_variables() -> Option<String> {
+fn retrieve_argc_variables() -> Option<String> {
     let value = env::var("ARGC_VARS").ok()?;
     let value = general_purpose::STANDARD.decode(value).ok()?;
     String::from_utf8(value).ok()
 }
 
-fn get_argc_help() -> String {
+fn get_argc_help(runtime: NativeRuntime) -> Result<String> {
     let about = concat!(
         env!("CARGO_PKG_DESCRIPTION"),
         " - ",
         env!("CARGO_PKG_REPOSITORY")
     );
-    format!(
+    let values = argc::eval(
+        runtime,
+        ARGC_COMPLETION_SCRIPT,
+        &["argc".into(), "--help".into()],
+        None,
+        None,
+    )?;
+    let argc_options = argc::ArgcValue::to_bash(&values);
+    let argc_options = argc_options
+        .lines()
+        .map(|v| v.trim())
+        .filter(|v| v.starts_with("--argc-"))
+        .map(|v| format!("    argc {v}\n"))
+        .collect::<Vec<String>>()
+        .join("");
+    let output = format!(
         r###"{about}
 
 USAGE:
-    argc --argc-eval <SCRIPT> [ARGS]...             Use `eval "$(argc --argc-eval "$0" "$@")"`
-    argc --argc-create [RECIPES]...                 Create a boilerplate argcfile
-    argc --argc-build <SCRIPT> [OUTPATH]            Generate bashscript without argc dependency
-    argc --argc-mangen <SCRIPT> <OUTDIR>            Generate man pages
-    argc --argc-completions <SHELL> [CMDS]...       Generate shell completion scripts
-    argc --argc-compgen <SHELL> <SCRIPT> <ARGS>...  Generate completion candidates
-    argc --argc-export <SCRIPT>                     Export command line definitions as json
-    argc --argc-parallel <SCRIPT> <ARGS>...         Run functions in parallel
-    argc --argc-script-path                         Print current argcfile path
-    argc --argc-shell-path                          Print current shell path
-    argc --argc-help                                Print help information
-    argc --argc-version                             Print version information
-"###
-    )
+{argc_options}"###
+    );
+    Ok(output)
 }
 
 fn get_argc_version() -> String {
@@ -288,10 +372,16 @@ fn get_argc_version() -> String {
     format!("{name} {version}")
 }
 
+fn get_term_width() -> Option<usize> {
+    env::var("TERM_WIDTH").ok().and_then(|v| v.parse().ok())
+}
+
 fn export_dir_vars(path: &str) -> String {
     if let (Some(argc_script_dir), Some(cwd)) = (
         get_argc_script_dir(path),
-        env::var("ARGC_PWD").ok().or_else(get_current_dir),
+        env::var("ARGC_PWD")
+            .ok()
+            .or_else(|| env::current_dir().ok().map(|v| v.to_string_lossy().into())),
     ) {
         let cd = if argc_script_dir != cwd {
             format!("cd {}\n", escape_shell_words(&argc_script_dir))
@@ -305,7 +395,7 @@ fn export_dir_vars(path: &str) -> String {
 }
 
 fn get_argc_script_dir(path: &str) -> Option<String> {
-    if candidate_script_names().iter().all(|v| !path.ends_with(v)) {
+    if runner_script_names().iter().all(|v| !path.ends_with(v)) {
         return None;
     }
     let path = Path::new(path);
@@ -319,19 +409,20 @@ fn get_argc_script_dir(path: &str) -> Option<String> {
     Some(script_dir.display().to_string())
 }
 
-fn parse_script_args(args: &[String]) -> Result<(String, Vec<String>)> {
+fn parse_script_args(args: &[String]) -> Result<(String, String, Vec<String>)> {
     if args.is_empty() {
-        bail!("No script provided");
+        bail!("No script file provided");
     }
     let script_file = args[0].as_str();
+    let script_file = normalize_script_path(script_file);
     let args: Vec<String> = args[1..].to_vec();
-    let source = fs::read_to_string(script_file)
+    let source = fs::read_to_string(&script_file)
         .with_context(|| format!("Failed to load script at '{}'", script_file))?;
-    let name = get_script_name(script_file)?;
+    let name = get_script_name(&script_file)?;
     let name = name.strip_suffix(".sh").unwrap_or(name);
     let mut cmd_args = vec![name.to_string()];
     cmd_args.extend(args);
-    Ok((source, cmd_args))
+    Ok((source, script_file, cmd_args))
 }
 
 fn get_script_name(script_path: &str) -> Result<&str> {
@@ -369,10 +460,10 @@ eval "$(argc --argc-eval "$0" "$@")"
 }
 
 fn get_script_path(recursive: bool) -> Option<(PathBuf, PathBuf)> {
-    let candidates = candidate_script_names();
+    let names = runner_script_names();
     let mut dir = env::current_dir().ok()?;
     loop {
-        for name in candidates.iter() {
+        for name in names.iter() {
             let path = dir.join(name);
             if path.exists() {
                 return Some((dir, path));
@@ -385,7 +476,7 @@ fn get_script_path(recursive: bool) -> Option<(PathBuf, PathBuf)> {
     }
 }
 
-fn candidate_script_names() -> Vec<String> {
+fn runner_script_names() -> Vec<String> {
     let mut names = vec![];
     if let Ok(name) = env::var("ARGC_SCRIPT_NAME") {
         names.push(name.clone());
@@ -397,8 +488,20 @@ fn candidate_script_names() -> Vec<String> {
     names
 }
 
+fn is_runner_script(script_file: &str) -> bool {
+    let name = match get_script_name(script_file) {
+        Ok(v) => v,
+        Err(_) => return false,
+    };
+    let name = name.strip_suffix(".sh").unwrap_or(name);
+    let expect_name = env::var("ARGC_SCRIPT_NAME")
+        .map(|v| v.to_lowercase())
+        .unwrap_or_else(|_| "argcfile".to_string());
+    name.to_lowercase() == expect_name
+}
+
 fn search_completion_script(args: &mut Vec<String>) -> Option<PathBuf> {
-    let search_paths = std::env::var("ARGC_COMPLETIONS_PATH").ok()?;
+    let search_paths = env::var("ARGC_COMPLETIONS_PATH").ok()?;
     let cmd = Path::new(&args[4])
         .file_stem()
         .and_then(|v| v.to_str())?
@@ -520,3 +623,17 @@ fn set_permissions<T: AsRef<Path>>(path: T) -> Result<()> {
 fn set_permissions<T: AsRef<Path>>(_path: T) -> Result<()> {
     Ok(())
 }
+
+fn normalize_script_path(path: &str) -> String {
+    if cfg!(windows)
+        && env::var("MSYSTEM").is_ok()
+        && path.len() >= 3
+        && path.starts_with('/')
+        && path.chars().nth(1).map(|v| v.is_ascii_alphabetic()) == Some(true)
+        && path.chars().nth(2) == Some('/')
+    {
+        let drive = path.chars().nth(1).unwrap().to_uppercase();
+        return format!("{}:{}", drive, &path[2..].replace('/', "\\"));
+    }
+    path.to_string()
+}
