@@ -14,6 +14,7 @@ pub(crate) trait Param {
     fn tag_name(&self) -> &str;
     fn guard(&self) -> Result<()>;
     fn multiple_values(&self) -> bool;
+    #[allow(unused)]
     fn render_source(&self) -> String;
 
     fn describe_oneline(&self) -> &str {
@@ -212,6 +213,7 @@ impl FlagOptionParam {
         }
     }
 
+    #[cfg(feature = "export")]
     pub(crate) fn export(&self) -> FlagOptionValue {
         FlagOptionValue {
             id: self.id().to_string(),
@@ -271,24 +273,24 @@ impl FlagOptionParam {
         let len = self.notations.len();
         if self.terminated()
             || self
-                .notation_modifer()
+                .notation_modifier()
                 .map(|v| matches!(v, '*' | '+'))
                 .unwrap_or_default()
         {
-            let min = if self.notation_modifer() == Some('*') {
+            let min = if self.notation_modifier() == Some('*') {
                 len - 1
             } else {
                 len
             };
             (min, MAX_ARGS)
-        } else if self.notation_modifer() == Some('?') {
+        } else if self.notation_modifier() == Some('?') {
             (len - 1, len)
         } else {
             (len, len)
         }
     }
 
-    pub(crate) fn notation_modifer(&self) -> Option<char> {
+    pub(crate) fn notation_modifier(&self) -> Option<char> {
         self.notations
             .last()
             .and_then(|name| ['*', '+', '?'].into_iter().find(|v| name.ends_with(*v)))
@@ -481,6 +483,7 @@ impl FlagOptionParam {
     }
 }
 
+#[cfg(feature = "export")]
 #[derive(Debug, Serialize)]
 pub struct FlagOptionValue {
     pub id: String,
@@ -571,6 +574,7 @@ impl PositionalParam {
         }
     }
 
+    #[cfg(feature = "export")]
     pub(crate) fn export(&self) -> PositionalValue {
         PositionalValue {
             id: self.id().to_string(),
@@ -622,6 +626,7 @@ impl PositionalParam {
     }
 }
 
+#[cfg(feature = "export")]
 #[derive(Debug, Serialize)]
 pub struct PositionalValue {
     pub id: String,
@@ -664,7 +669,10 @@ impl Param for EnvParam {
     }
 
     fn guard(&self) -> Result<()> {
-        if !matches!(self.data().modifer, Modifier::Optional | Modifier::Required) {
+        if !matches!(
+            self.data().modifier,
+            Modifier::Optional | Modifier::Required
+        ) {
             bail!("can only be a single value")
         }
         if self.data.env.is_some() {
@@ -700,6 +708,7 @@ impl EnvParam {
         }
     }
 
+    #[cfg(feature = "export")]
     pub(crate) fn export(&self) -> EnvValue {
         EnvValue {
             id: self.id().to_string(),
@@ -731,6 +740,7 @@ impl EnvParam {
     }
 }
 
+#[cfg(feature = "export")]
 #[derive(Debug, Serialize)]
 pub struct EnvValue {
     pub id: String,
@@ -747,7 +757,7 @@ pub(crate) struct ParamData {
     pub(crate) describe: String,
     pub(crate) choice: Option<ChoiceValue>,
     pub(crate) default: Option<DefaultValue>,
-    pub(crate) modifer: Modifier,
+    pub(crate) modifier: Modifier,
     pub(crate) env: Option<Option<String>>,
 }
 
@@ -758,28 +768,28 @@ impl ParamData {
             describe: String::new(),
             choice: None,
             default: None,
-            modifer: Modifier::Optional,
+            modifier: Modifier::Optional,
             env: None,
         }
     }
 
     pub(crate) fn required(&self) -> bool {
-        self.modifer.required() && self.default.is_none()
+        self.modifier.required() && self.default.is_none()
     }
 
     pub(crate) fn multiple(&self) -> bool {
-        self.modifer.multiple()
+        self.modifier.multiple()
     }
 
     pub(crate) fn args_delimiter(&self) -> Option<char> {
-        match &self.modifer {
+        match &self.modifier {
             Modifier::DelimiterRequired(c) | Modifier::DelimiterOptional(c) => Some(*c),
             _ => None,
         }
     }
 
     pub(crate) fn terminated(&self) -> bool {
-        matches!(self.modifer, Modifier::Terminated)
+        matches!(self.modifier, Modifier::Terminated)
     }
 
     pub(crate) fn choice_fn(&self) -> Option<(&String, &bool)> {
@@ -820,7 +830,7 @@ impl ParamData {
 
     pub(crate) fn render_source_of_name_value(&self, name_suffix: &str) -> String {
         let mut output = format!("{}{name_suffix}", self.name);
-        output.push_str(&self.modifer.render());
+        output.push_str(&self.modifier.render());
         match (&self.choice, &self.default) {
             (Some(ChoiceValue::Values(values)), None) => {
                 output.push_str(&format!("[{}]", Self::render_choice_values(values)));
@@ -845,23 +855,29 @@ impl ParamData {
 
     pub(crate) fn render_describe(&self, describe: &str, id: &str) -> String {
         let mut output = describe.to_string();
-        let multiline = output.contains('\n');
+        let multiline = describe.contains('\n');
+        let mut documented = false;
         let sep = if multiline { '\n' } else { ' ' };
-        if let Some(DefaultValue::Value(value)) = &self.default {
-            if !output.is_empty() {
-                output.push(sep)
-            }
-            output.push_str(&format!("[default: {}]", escape_shell_words(value)));
-        }
         if let Some(ChoiceValue::Values(values)) = &self.choice {
-            if !output.is_empty() {
-                output.push(sep)
+            documented = values.iter().all(|v| describe.contains(v));
+            if !documented {
+                if !output.is_empty() {
+                    output.push(sep)
+                }
+                let values: Vec<String> = values.iter().map(|v| escape_shell_words(v)).collect();
+                output.push_str(&format!("[possible values: {}]", values.join(", ")));
+            }
+        }
+        if !documented {
+            if let Some(DefaultValue::Value(value)) = &self.default {
+                if !output.is_empty() {
+                    output.push(sep)
+                }
+                output.push_str(&format!("[default: {}]", escape_shell_words(value)));
             }
-            let values: Vec<String> = values.iter().map(|v| escape_shell_words(v)).collect();
-            output.push_str(&format!("[possible values: {}]", values.join(", ")));
         }
         if let Some(env) = self.normalize_bind_env(id) {
-            if !output.is_empty() {
+            if !describe.contains(&env) && !output.is_empty() {
                 output.push(sep)
             }
             output.push_str(&format!("[env: {env}]"));
