@@ -71,6 +71,20 @@ fn create_with_tasks() {
         .success();
 }
 
+#[test]
+fn run() {
+    let path_env_var = get_path_env_var();
+    let path = locate_script("examples/demo.sh");
+    Command::cargo_bin("argc")
+        .unwrap()
+        .arg("--argc-run")
+        .arg(path)
+        .env("PATH", path_env_var)
+        .assert()
+        .stderr(predicates::str::contains("USAGE: demo"))
+        .success();
+}
+
 #[test]
 fn build_stdout() {
     let path = locate_script("examples/demo.sh");
@@ -84,10 +98,10 @@ fn build_stdout() {
 }
 
 #[test]
-fn build_outpath() {
-    let path = locate_script("examples/demo.sh");
+fn run_build() {
+    let path = locate_script("examples/strict.sh");
     let tmpdir = tmpdir();
-    let outpath = tmpdir.join("demo.sh");
+    let outpath = tmpdir.join("strict.sh");
     Command::cargo_bin("argc")
         .unwrap()
         .arg("--argc-build")
@@ -95,8 +109,23 @@ fn build_outpath() {
         .arg(&outpath)
         .assert()
         .success();
-    let script = std::fs::read_to_string(&outpath).unwrap();
-    assert!(script.contains("# ARGC-BUILD"));
+
+    Command::cargo_bin("argc")
+        .unwrap()
+        .arg("--argc-run")
+        .arg(&outpath)
+        .args([
+            "--fa",
+            "--oa",
+            "oa1",
+            "--of=of1,of2",
+            "--oca=a",
+            "--ofa",
+            "abc",
+        ])
+        .assert()
+        .stdout(predicates::str::contains("argc__fn=main"))
+        .success();
 }
 
 #[test]
@@ -168,6 +197,22 @@ fn compgen_argc() {
         .success();
 }
 
+#[test]
+fn compgen_kind() {
+    Command::cargo_bin("argc")
+        .unwrap()
+        .args([
+            "--argc-compgen",
+            "fish",
+            argc::COMPGEN_KIND_SYMBOL,
+            "shell",
+            "",
+        ])
+        .assert()
+        .stdout(predicates::str::contains("zsh"))
+        .success();
+}
+
 #[test]
 fn export() {
     let path = locate_script("examples/options.sh");
